# Chapter7.py

# Pythonic -> resource allocation, readability, framework
# 1. Python Syntax -> 2. Pythonic "what does this solve for"

# Indentation process using 4 spaces or the tab key
# readability
def indentation():
    # Python syntax 4 spaces or tab
    print() # if you use 4 spaces, it may cause copy/paste issues
    # Pythonic -> ALWAYS use tab or shift/tab to move code blocks
    #  print() -> this will cause a tab indent error, BUT if you always use the tab key, this will never be a problem

# Variable Declaration
print("=== Variable Declaration ===")
# Python
x = 7 # individual variables are declared on their own line
y = 3 # individual variables are declared on their own line
z = x + y
print(z)
# Pythonic
# variables are grouped in a single line
x,y = 7,3
z = x + y
print(z)
firstName, lastName = 'Bob', 'Smith' # input(), input()
a,b = firstName, lastName
fullName = a + ' ' + b
print(fullName)

print("==========================")
print("=== List Comprehension ===")
data = ['Bob', 'Joe', 'Mary'] # len(3) range[0:3] startsWith[0]: endsAt[3]

# Non Pythonic loop -> for x in range(0, len(dataset))
for i in range(0, len(data)):
    if i == 2: # we should not see Mary
        break
    print(i) # index number 
    print(data[i]) # Bob

print("===========")
# Pythonic loop -> for x in ys # this has a natural len() and range() with a break/continue
# before you product the views data, you should already have prepped the list
# original data 2,000 rows [data]
# query out customers where state = 'FL' TOP 10 -> List[data]<10 rows>
# remove whitespaces with strip(), names title() BOb -> Bob
# then for x in data -> view
for x in data:
    print(x)  # x is the value of date -> Bob

print("==============")

print("=== EAFP and LBYL concepts")
# LBYL -> Look Before You Leap -> Non Pythonic Level 1 Python Syntax
# EAFP -> Easier to Ask for Forgiveness than Permission -> Pythonic
# https://en.wikipedia.org/wiki/EAFP

# control variable
j = "Bob"

# Python 
# conditional test and expressions and operations using an if keyword
# LBYL -> else keyword -> TRUE / FALSE
if j == 'Bob':
    print("This is the True part of the return")
else:
    print("This is the False part of the return. LBYL")

# Pythonic EAFP -> try except finally(optional)
# EAFP  TRUE / don't do it
# while True: do something
# related to a switch True: case
# do while: TRUE
# try keyword ONLY try TRUE
# use the if keyword WITHOUT the else
try:  # chapter 8
    # every line of code to the end of line and if it passes, do the line, failes -> except keyword
    print() # try this -> pass or fail
    print() # try this -> pass or fail
    x,y = 7,3 # try this -> pass or fail
    z = x + y # try this -> pass or fail
    if j == 'Bob': # try this -> pass or fail
        print("do something") # try this -> pass or fail
    # NO else -> EAFP
    h = z / 0 # # try this -> pass or fail ZeroDivisionError Exception
    
except Exception:  # Im sorry, please forgive me
    print()

finally:
    print("This runs either Pass<Exit Code 0> or Fail<Exit Code 1>")

print("=====================")

print("=== Context and Connection Managers for IO Classes")
# open() function from Chapter 4 Level 1

txtFile = 'LogFile.txt' # IOFile example

# Python Syntax
# defineVariable = open(IOFile)
# ASSIGNING an object to an IO in memory
# requires the use of the close() to close the connection
file = open(txtFile, 'w')  # open a connection to a file as store in memory
content = file.write("Data to write into the file")
file.close() # close the connection

# Pythonic Syntax
# with open(IOFile) as namespace/alias<var in the scope of open()>
# Using the with as keywords to create a local variable in a scope
# close() is not needed as a data streams are one way
with open(txtFile, 'w') as file:
    content = file.write("Data written using the data stream")

print("========================")

print("=== Regular Expressions ===")
# There really is no Python here, regular expressions (like lambda expressions we learned in level 1) already handle being Pythonic

# lambda expressions
# anonyous singular function
# variable = lambda<keyword> params: singular expression or operation

addThreeNumbers = lambda n1, n2, n3: n1 + n2 + n3

print(
    addThreeNumbers(3, 5, 9) # call the lambda function, pass 3 args
) # print

# regex101.com
# regexlib.com

emailAddress = input("Enter your email address: ")

# re is a Python Standard Library
import re

pattern = "[\w\.-]+@[\w\.-]+\.[\w]{2,8}"

match = re.search(pattern,emailAddress) # True

if match:  # LBYL
    print("You entered a valid email address")
else:
    print("You entered an invalid email address.")


# EOF